/* global imce:true */
(function ($, Drupal, imce) {
  'use strict';

  /**
   * @file
   * Defines Search plugin for IMCE.
   */

  /**
   * Sanitize strings to prevent XSS.
   */
  imce.sanitizeString = function(str) {
    if (!str) return '';
    return str.toString()
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;');
  };

  /**
   * Init handler for Search.
   */
  imce.bind('init', imce.searchInit = function () {
    // Add toolbar button.
    imce.addTbb('search', {
      title: Drupal.t('Search'),
      content: imce.createSearchForm(),
      icon: 'search'
    });

    // Add custom CSS for search results and highlight
    var style = document.createElement('style');
    style.textContent = `
      .imce-search-form {
        padding: 10px;
        min-width: 400px;
      }
      .imce-search-results {
        max-height: 400px;
        overflow-y: auto;
        margin-top: 10px;
        border-top: 1px solid #ddd;
      }
      .imce-search-list {
        list-style: none;
        padding: 0;
        margin: 0;
      }
      .imce-search-item {
        padding: 8px;
        cursor: pointer;
        border-bottom: 1px solid #ddd;
        display: flex;
        align-items: center;
      }
      .imce-search-item:hover {
        background-color: #f5f5f5;
      }
      .imce-search-item-preview {
        width: 50px;
        height: 50px;
        margin-right: 10px;
        background-size: cover;
        background-position: center;
        background-repeat: no-repeat;
        flex-shrink: 0;
      }
      .imce-search-item-info {
        flex-grow: 1;
      }
      .imce-search-item-name {
        font-weight: bold;
        display: block;
      }
      .imce-search-item-details {
        font-size: 0.9em;
        color: #666;
      }
      .imce-search-no-results {
        padding: 10px;
        color: #666;
        font-style: italic;
      }
      .imce-search-count {
        margin: 5px 0;
        padding: 5px;
        background: #f5f5f5;
        border-radius: 3px;
      }
      .imce-search-loading {
        text-align: center;
        padding: 20px;
      }
      .imce-highlight-search {
        background-color: #ffeb3b !important;
        border: 2px solid #ffc107 !important;
      }
    `;
    document.head.appendChild(style);
  });

  /**
   * Creates search form.
   */
  imce.createSearchForm = function () {
    var form = imce.searchForm;

    if (!form) {
      form = imce.searchForm = imce.createEl('<form class="imce-search-form">' +
          '<div class="imce-form-item">' +
          '<label>' + Drupal.t('Search files') + '</label>' +
          '<input type="text" name="keywords" size="30" maxlength="50" />' +
          '</div>' +
          '<div class="imce-form-actions">' +
          '<button type="submit" name="op" class="imce-search-button">' + Drupal.t('Search') + '</button>' +
          '</div>' +
          '<div class="imce-search-count"></div>' +
          '<div class="imce-search-results"></div>' +
          '</form>');
      form.onsubmit = imce.eSearchSubmit;

      var els = form.elements;
      els.keywords.placeholder = Drupal.t('Enter keywords to search');
    }

    return form;
  };

  /**
   * Submit event for search form.
   */
  imce.eSearchSubmit = function () {
    var form = this;
    var $form = $(form);
    var $results = $form.find('.imce-search-results');
    var $count = $form.find('.imce-search-count');
    var keywords = form.elements.keywords.value;

    // Don't search if keywords are empty
    if (!keywords.trim()) {
      return false;
    }

    // Clear previous results
    $results.empty();
    $count.empty();

    // Show loading
    $results.html('<div class="imce-search-loading">' + Drupal.t('Searching...') + '</div>');

    // Send search request with keywords as a string
    imce.ajax('search', {data: {keywords: keywords}}).done(function(response) {
      if (response.data && response.data.length) {
        // Update count
        $count.html(Drupal.formatPlural(
          response.data.length,
          '1 item found',
          '@count items found'
        ));

        // Build results
        var resultsHtml = '<ul class="imce-search-list">';
        response.data.forEach(function(item) {
          var isImage = item.type && item.type.startsWith('image/');
          var preview = isImage 
            ? '<div class="imce-search-item-preview" style="background-image: url(\'' + item.url + '\')"></div>'
            : '<div class="imce-search-item-preview"><span aria-hidden="true">📄</span></div>';
          
          var size = item.size ? imce.formatSize(item.size) : '';
          var date = item.changed ? new Date(item.changed * 1000).toLocaleDateString() : '';
          
          // Extract the directory path from the URL
          var urlParts = item.url.split('/');
          var fileName = urlParts.pop();
          // Get everything after 'files' in the path
          var dirIndex = urlParts.indexOf('files');
          var dirPath = dirIndex !== -1 ? urlParts.slice(dirIndex + 1).join('/') : '';
          // If we're in the root of files, use '.'
          dirPath = dirPath || '.';
          
          // Store debug info
          var debugInfo = {
            url: item.url,
            dirPath: dirPath,
            fileName: fileName,
            originalPath: item.url
          };
          
          resultsHtml += '<li class="imce-search-item" data-url="' + item.url + 
            '" data-path="' + dirPath + 
            '" data-filename="' + fileName + 
            '" data-debug="' + encodeURIComponent(JSON.stringify(debugInfo)) + '">' +
            preview +
            '<div class="imce-search-item-info">' +
            '<span class="imce-search-item-name">' + item.filename + '</span>' +
            '<span class="imce-search-item-details">' +
            (size ? size + ' - ' : '') +
            (date ? date : '') +
            '</span>' +
            '</div>' +
            '</li>';
        });
        resultsHtml += '</ul>';
        $results.html(resultsHtml);

        // Add click handler
        $results.find('.imce-search-item').on('click', function() {
          var $item = $(this);
          var debugInfo = JSON.parse(decodeURIComponent($item.data('debug')));
          var fileName = decodeURIComponent(debugInfo.fileName);
          var folderPath = decodeURIComponent(debugInfo.dirPath);

          // Close the search popup
          imce.getTbb('search').closePopup();

          // Get the folder directly
          var targetFolder = imce.addFolder(folderPath);
          
          if (targetFolder) {
            // Set as active folder immediately
            imce.activeFolder = targetFolder;

            // If it has an open method, use it
            if (typeof targetFolder.open === 'function') {
              targetFolder.open();
            }
            // Or try load method
            else if (typeof targetFolder.load === 'function') {
              targetFolder.load();
            }

            // Function to check for and highlight the item
            function findAndHighlightItem() {
              const items = document.querySelectorAll('.imce-item');
              let found = false;
              
              items.forEach(item => {
                const nameEl = item.querySelector('.imce-item-name');
                if (nameEl && nameEl.textContent === fileName) {
                  // Remove highlight from any previously highlighted items
                  document.querySelectorAll('.imce-highlight-search').forEach(el => {
                    el.classList.remove('imce-highlight-search');
                  });
                  // Add highlight to the matching item
                  item.classList.add('imce-highlight-search');
                  // Scroll the item into view smoothly
                  item.scrollIntoView({ 
                    behavior: 'smooth',
                    block: 'center'
                  });
                  found = true;
                }
              });
              return found;
            }

            // Set up polling with timeout
            let attempts = 0;
            const maxAttempts = 30; // 3 seconds total (30 * 100ms)
            let pollInterval;

            function pollForItem() {
              if (findAndHighlightItem()) {
                clearInterval(pollInterval);
              } else if (++attempts >= maxAttempts) {
                clearInterval(pollInterval);
              }
            }

            // Start polling after initial delay
            setTimeout(() => {
              pollInterval = setInterval(pollForItem, 100);
            }, 500);
          }
        });
      } else {
        $results.html('<div class="imce-search-no-results">' + Drupal.t('No files found.') + '</div>');
      }
    });

    return false;
  };

  /**
   * Format file size in human readable format.
   */
  imce.formatSize = function(bytes) {
    if (bytes === 0) return '0 B';
    var k = 1024;
    var sizes = ['B', 'KB', 'MB', 'GB'];
    var i = Math.floor(Math.log(bytes) / Math.log(k));
    return parseFloat((bytes / Math.pow(k, i)).toFixed(1)) + ' ' + sizes[i];
  };

})(jQuery, Drupal, imce);
