<?php

namespace Drupal\imce_search_plugin\Plugin\ImcePlugin;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\imce\Imce;
use Drupal\imce\ImceFM;
use Drupal\imce\ImcePluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines Imce Search plugin.
 *
 * @ImcePlugin(
 *   id = "search",
 *   label = @Translation("Search"),
 *   operations = {
 *     "search" = "opSearch"
 *   }
 * )
 */
class Search extends ImcePluginBase implements ContainerFactoryPluginInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a Search object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Connection $database, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->database = $database;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('database'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildPage(array &$page, ImceFM $fm) {
    $page['#attached']['library'][] = 'imce_search_plugin/drupal.imce.search';
  }

  /**
   * Operation handler: search.
   */
  public function opSearch(ImceFM $fm) {
    $keywords = $fm->request->request->get('keywords', '');
    $files = [];

    if (!empty($keywords)) {
      try {
        // Use proper database query builder with placeholders.
        $query = $this->database->select('file_managed', 'fm');
        $query->leftJoin('file_usage', 'fu', 'fm.fid = fu.fid');
        
        $query->fields('fm', ['fid', 'filename'])
          ->condition('fm.filename', '%' . $this->database->escapeLike($keywords) . '%', 'LIKE')
          ->groupBy('fm.fid')
          ->groupBy('fm.filename')
          ->orderBy('fm.changed', 'DESC')
          ->range(0, 50);

        $result = $query->execute();

        foreach ($result as $record) {
          try {
            $file = $this->entityTypeManager->getStorage('file')->load($record->fid);
            if ($file) {
              $files[] = [
                'filename' => $file->getFilename(),
                'url' => $file->createFileUrl(FALSE),
                'type' => $file->getMimeType(),
                'size' => $file->getSize(),
                'changed' => $file->getChangedTime(),
              ];
            }
          }
          catch (\Exception $e) {
            watchdog_exception('imce_search_plugin', $e);
          }
        }
      }
      catch (\Exception $e) {
        watchdog_exception('imce_search_plugin', $e);
      }
    }

    $fm->addResponse('data', $files);
  }

}
